'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ComboBox = undefined;

var _babelHelperVueJsxMergeProps = require('babel-helper-vue-jsx-merge-props');

var _babelHelperVueJsxMergeProps2 = _interopRequireDefault(_babelHelperVueJsxMergeProps);

var _from = require('babel-runtime/core-js/array/from');

var _from2 = _interopRequireDefault(_from);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ComboBox = exports.ComboBox = {
  name: 'multi-select-value-input',
  props: {
    id: { type: [String], required: false, default: 'multi-select-input' },
    open: { type: [Boolean], required: false, default: false },
    options: { type: [Array], required: false, default: function _default() {
        return new Array();
      } },
    preSelectedOptions: { type: [Array], required: false, default: function _default() {
        return new Array();
      } },
    disabled: { type: [Boolean], required: false },
    placeholder: { type: [String], required: false, default: 'Search' },
    multiple: { type: [Boolean], required: false, default: false },
    closeOnSelect: { type: [Boolean], required: false, default: true },
    optionKey: { type: [String], required: false, default: 'id' }, // The object key you want to use when filtering and comparing, a uniqe identifier
    width: { type: [String], required: false, default: '10em' },
    customAttrs: { type: [Object], required: false, default: function _default() {
        return new Object();
      } }
  },
  data: function data() {
    return {
      value: '',
      multiSelectOpen: true,
      filteredOptions: [],
      selectedOptions: [],
      selectedIndex: -1 //default set to not select any, if the user presses arrow down the first one will be selected
    };
  },
  beforeMount: function beforeMount() {
    // Doing our own immutable handler here. Read in the values from parent but keep our own instance of the selected values
    // this.filteredOptions = Array.from(this.$props.options)
    this.filteredOptions = (0, _from2.default)(this.$props.options);
    this.multiSelectOpen = this.$props.open;

    if (this.$props.preSelectedOptions && this.$props.preSelectedOptions.length > 0) {
      this.selectedOptions = (0, _from2.default)(this.$props.preSelectedOptions);
    }
  },
  mounted: function mounted() {
    this.setFocusToMultiSelect();
  },

  methods: {
    // Want to allow to either pass in an array of strings as options or an array of arbitrary objects with a specified key to validate with (this.$props.optionKey)
    selectedOptionMatches: function selectedOptionMatches(option, selectedOption) {
      var optionKey = this.$props.optionKey;

      if (typeof option === 'string' && typeof selectedOption === 'string') {
        return option === selectedOption;
      } else {
        return option[optionKey] === selectedOption[optionKey];
      }
    },
    setFocusToMultiSelect: function setFocusToMultiSelect() {
      document.getElementById('multi-select-' + this.$props.id).focus();
    },
    returnSelected: function returnSelected() {
      this.$listeners.inputHandler(this.selectedOptions);
    },
    toggleDropdown: function toggleDropdown() {
      this.multiSelectOpen = !this.multiSelectOpen;
    },
    closeDropdown: function closeDropdown(event) {
      this.multiSelectOpen = false;
      if (this.selectedOptions.length > 0) {
        this.returnSelected();
      }
    },
    openDropdown: function openDropdown() {
      this.multiSelectOpen = true;
    },
    setSelectedIndex: function setSelectedIndex(index) {
      this.selectedIndex = index;
    },
    typeAheadUp: function typeAheadUp() {
      if (this.selectedIndex === 0) {
        this.selectedIndex = this.filteredOptions.length - 1;
      } else {
        this.selectedIndex = this.selectedIndex - 1;
      }
    },
    typeAheadDown: function typeAheadDown() {
      if (this.selectedIndex < this.filteredOptions.length - 1) {
        this.selectedIndex = this.selectedIndex + 1;
      } else {
        this.selectedIndex = 0;
      }
    },
    selectedOptionsContains: function selectedOptionsContains(option) {
      var _this = this;

      var optionKey = this.optionKey;

      var hit = false;
      this.selectedOptions.forEach(function (opt) {
        if (_this.selectedOptionMatches(opt, option)) {
          hit = true;
        }
      });
      return hit;
    },
    removeFromSelected: function removeFromSelected(opt) {
      var _this2 = this;

      var optionKey = this.$props.optionKey;

      this.selectedOptions.forEach(function (selectedOpt, i) {
        if (_this2.selectedOptionMatches(opt, selectedOpt)) {
          _this2.selectedOptions.splice(i, 1);
        }
      });
    },
    handleSelect: function handleSelect() {
      var _$props = this.$props,
          multiple = _$props.multiple,
          closeOnSelect = _$props.closeOnSelect,
          optionKey = _$props.optionKey;

      var option = this.filteredOptions[this.selectedIndex];
      if (this.selectedOptionsContains(option)) {
        this.removeFromSelected(option);
      } else {
        if (this.$props.multiple) {
          this.selectedOptions.push(option);
        } else {
          this.value = option[optionKey] || option;
          this.selectedOptions = [option];
        }
      }

      if (multiple || !closeOnSelect) {
        this.setFocusToMultiSelect();
      } else {
        this.closeDropdown();
      }
    },
    filterOptions: function filterOptions(value) {
      var optionKey = this.$props.optionKey;

      if (value !== "") {
        this.filteredOptions = this.$props.options.filter(function (option) {
          var checkProperty = null;
          if (option[optionKey] && typeof option[optionKey] === 'string') {
            checkProperty = option[optionKey];
          } else if (option.id && typeof option.id === 'string') {
            checkProperty = option.id;
          } else {
            checkProperty = option.id.header ? option.id.header : '';
          }
          return checkProperty.toLowerCase().indexOf(value.toLowerCase()) !== -1;
        });
      } else {
        this.filteredOptions = this.$props.options;
      }
    },
    handlePressedKey: function handlePressedKey(key) {
      switch (key) {
        case 'ArrowDown':
          return this.typeAheadDown();
        case 'ArrowUp':
          return this.typeAheadUp();
        case 'Escape':
          return this.closeDropdown();
        case 'Enter':
          return this.handleSelect();
      }
    },
    keyUpHandler: function keyUpHandler(value, event) {
      this.value = value;
      this.openDropdown();
      this.filterOptions(value);
      this.handlePressedKey(event.code || event.key);
      if (this.$listeners.keyUpHandler) {
        this.$listeners.keyUpHandler(event);
      }
    },
    handleInputBlur: function handleInputBlur(event) {
      if (!event.relatedTarget) {
        this.closeDropdown();
      }
    },
    renderOptions: function renderOptions(h) {
      var _this3 = this;

      var filteredOptions = this.filteredOptions;
      var optionKey = this.$props.optionKey;

      return filteredOptions.map(function (opt, index) {
        var classes = 'multi-select-option';
        if (index === _this3.selectedIndex) {
          classes += ' highlight';
        }

        _this3.selectedOptions.forEach(function (selectedOpt) {
          if (_this3.selectedOptionMatches(opt, selectedOpt)) {
            // if (opt[optionKey] === selectedOpt[optionKey]) {
            classes += ' multi-selected';
          }
        });

        return h(
          'li',
          (0, _babelHelperVueJsxMergeProps2.default)([{ 'class': classes, attrs: { tabindex: index }
          }, {
            on: {
              'mouseover': function mouseover($event) {
                for (var _len = arguments.length, attrs = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                  attrs[_key - 1] = arguments[_key];
                }

                _this3.setSelectedIndex.bind(_this3, index).apply(undefined, [$event].concat(attrs));
              },
              'click': function click($event) {
                for (var _len2 = arguments.length, attrs = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
                  attrs[_key2 - 1] = arguments[_key2];
                }

                _this3.handleSelect.apply(_this3, [$event].concat(attrs));
              }
            }
          }]),
          [h('a', [opt[optionKey] || opt])]
        );
      });
    }
  },
  watch: {
    options: function options(newOptions, oldOptions) {
      // Don't re-populate the options array if nothing new has come in. We check if the key of the first item are the same
      if (newOptions.length !== oldOptions.length || newOptions[0] && newOptions[0][this.$props.optionKey] !== oldOptions[0][this.$props.optionKey]) {
        this.selectedOptions = [];
        this.filteredOptions = (0, _from2.default)(newOptions);
        this.value = '';
      }
    },
    preSelectedOptions: function preSelectedOptions(preOptions) {
      this.selectedOptions = (0, _from2.default)(preOptions);
    },
    open: function open(_open) {
      var _this4 = this;

      this.multiSelectOpen = _open;
      if (_open) {
        setTimeout(function () {
          _this4.setFocusToMultiSelect();
        }, 200);
      }
    }
  },
  render: function render(h) {
    var _this5 = this;

    var _$props2 = this.$props,
        id = _$props2.id,
        header = _$props2.header,
        options = _$props2.options,
        placeholder = _$props2.placeholder,
        selectedOptions = _$props2.selectedOptions,
        closeOnSelect = _$props2.closeOnSelect,
        width = _$props2.width,
        customAttrs = _$props2.customAttrs;

    var attributes = {
      attrs: customAttrs
    };

    return h(
      'div',
      {
        attrs: { id: id },
        'class': 'multi-select-dropdown' },
      [h(
        'div',
        (0, _babelHelperVueJsxMergeProps2.default)([{ 'class': 'multi-select-toggle clear-fix' }, {
          on: {
            'click': function click($event) {
              for (var _len3 = arguments.length, attrs = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
                attrs[_key3 - 1] = arguments[_key3];
              }

              _this5.toggleDropdown.apply(_this5, [$event].concat(attrs));
            }
          }
        }]),
        [h('input', (0, _babelHelperVueJsxMergeProps2.default)([{
          attrs: {
            autocomplete: 'off',

            id: 'multi-select-' + id,
            type: 'text',

            placeholder: placeholder
          },
          style: 'width: ' + width, domProps: {
            'value': this.value
          }
        }, attributes, {
          on: {
            'blur': function blur($event) {
              for (var _len4 = arguments.length, attrs = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
                attrs[_key4 - 1] = arguments[_key4];
              }

              _this5.handleInputBlur.apply(_this5, [$event].concat(attrs));
            },
            'keyup': function keyup($event) {
              for (var _len5 = arguments.length, attrs = Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
                attrs[_key5 - 1] = arguments[_key5];
              }

              (function (e) {
                return _this5.keyUpHandler(e.target.value, e);
              }).apply(undefined, [$event].concat(attrs));
            }
          }
        }])), h('i', {
          attrs: { role: 'presentation' },
          'class': 'open-indicator' })]
      ), this.multiSelectOpen && h(
        'ul',
        {
          attrs: { tabindex: '0', id: 'multi-select-list-' + id },
          'class': 'dropdown-menu' },
        [this.renderOptions(h)]
      )]
    );
  }
};