'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ComboButton = undefined;

var _babelHelperVueJsxMergeProps = require('babel-helper-vue-jsx-merge-props');

var _babelHelperVueJsxMergeProps2 = _interopRequireDefault(_babelHelperVueJsxMergeProps);

var _themeBuilder = require('../utils/themeBuilder');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ComboButton = exports.ComboButton = {
  name: 'combo-button-component',
  props: {
    label: { type: [String], required: true },
    disabled: { type: [Boolean], required: false },
    isLoading: { type: [Boolean], required: false },
    additionalClasses: { type: [String], required: false, default: '' },
    options: { type: [Array], required: false, default: null },
    theme: { type: [String], required: false },
    customAttrs: { type: [Object], required: false, default: function _default() {
        return new Object();
      } }
  },
  beforeMount: function beforeMount() {
    if (this.$props.theme) {
      (0, _themeBuilder.setTheme)(this.$props.theme);
    }
  },

  data: function data() {
    return {
      showOptions: false,
      preventClose: false // There's a hierrarchy between onBlur and onClick, onBlur is called before. We want to allow holding the to mouse and still be able to see the dropdown and then call the callback.
    };
  },
  methods: {
    renderDropdown: function renderDropdown(h) {
      var _this = this;

      return h('div', (0, _babelHelperVueJsxMergeProps2.default)([{ 'class': 'combo-options', attrs: { tabindex: '0' }
      }, {
        on: {
          'click': function click($event) {
            for (var _len = arguments.length, attrs = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
              attrs[_key - 1] = arguments[_key];
            }

            (function () {
              return _this.showOptions = !_this.showOptions;
            }).apply(undefined, [$event].concat(attrs));
          },
          'blur': function blur($event) {
            for (var _len2 = arguments.length, attrs = Array(_len2 > 1 ? _len2 - 1 : 0), _key2 = 1; _key2 < _len2; _key2++) {
              attrs[_key2 - 1] = arguments[_key2];
            }

            _this.handleDropdownClose.apply(_this, [$event].concat(attrs));
          }
        }
      }]));
    },
    renderDropdownOptions: function renderDropdownOptions(h) {
      var _this2 = this;

      var _$props = this.$props,
          options = _$props.options,
          disabled = _$props.disabled;

      if (!disabled) {
        return h(
          'ul',
          { 'class': 'dropdown-container' },
          [options.map(function (opt, index) {
            return h(
              'li',
              (0, _babelHelperVueJsxMergeProps2.default)([{ key: index }, {
                on: {
                  'mousedown': function mousedown($event) {
                    for (var _len3 = arguments.length, attrs = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
                      attrs[_key3 - 1] = arguments[_key3];
                    }

                    (function () {
                      return _this2.preventClose = true;
                    }).apply(undefined, [$event].concat(attrs));
                  },
                  'click': function click($event) {
                    for (var _len4 = arguments.length, attrs = Array(_len4 > 1 ? _len4 - 1 : 0), _key4 = 1; _key4 < _len4; _key4++) {
                      attrs[_key4 - 1] = arguments[_key4];
                    }

                    _this2.handleOptionClick.bind(_this2, opt).apply(undefined, [$event].concat(attrs));
                  }
                }
              }]),
              [h('span', [opt.displayName]), opt.icon && h('img', {
                attrs: { src: opt.icon }
              })]
            );
          })]
        );
      }
    },
    handleDropdownClose: function handleDropdownClose() {
      if (!this.preventClose) {
        this.showOptions = false;
      }
    },
    handleOptionClick: function handleOptionClick(option, e) {
      option.callback();
      this.preventClose = false;
      this.handleDropdownClose();
    }
  },
  render: function render(h) {
    var _$props2 = this.$props,
        label = _$props2.label,
        disabled = _$props2.disabled,
        isLoading = _$props2.isLoading,
        additionalClasses = _$props2.additionalClasses,
        customAttrs = _$props2.customAttrs,
        options = _$props2.options;
    var clickHandler = this.$listeners.clickHandler;

    var attributes = {
      attrs: customAttrs
    };

    return h(
      'div',
      (0, _babelHelperVueJsxMergeProps2.default)([{
        attrs: { tabIndex: '0' },
        'class': 'combo-button ' + additionalClasses + ' ' + (disabled && 'disabled') + ' ' + (isLoading && 'loading') + ' ' + (options && 'with-options') }, attributes]),
      [h(
        'div',
        (0, _babelHelperVueJsxMergeProps2.default)([{ 'class': 'combo-main-content' }, {
          on: {
            'click': function click($event) {
              for (var _len5 = arguments.length, attrs = Array(_len5 > 1 ? _len5 - 1 : 0), _key5 = 1; _key5 < _len5; _key5++) {
                attrs[_key5 - 1] = arguments[_key5];
              }

              (!disabled && clickHandler).apply(undefined, [$event].concat(attrs));
            }
          }
        }]),
        [label]
      ), options && this.renderDropdown(h), this.showOptions && this.renderDropdownOptions(h)]
    );
  }
};